from django.contrib.gis.geos import Point

from location_field.forms.plain import PlainLocationField


class LocationField(PlainLocationField):
    def clean(self, value):
        try:
            lat, lng = value.split(',')
            return Point(float(lng), float(lat))
        except ValueError:
            return None



import six
import json

from django.conf import settings
from django import forms
from django.forms import widgets
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe


class LocationWidget(widgets.TextInput):
    def __init__(self, **kwargs):
        attrs = kwargs.pop('attrs', None)

        self.options = dict(settings.LOCATION_FIELD)
        self.options['field_options'] = {
            'based_fields': kwargs.pop('based_fields'),
        }

        super(LocationWidget, self).__init__(attrs)

    def render(self, name, value, attrs=None, renderer=None):
        if value is not None:
            try:
                if isinstance(value, six.string_types):
                    lat, lng = value.split(',')
                else:
                    lng = value.x
                    lat = value.y

                value = '%s,%s' % (
                    float(lat),
                    float(lng),
                )
            except ValueError:
                value = ''
        else:
            value = ''

        if '-' not in name:
            prefix = ''
        else:
            prefix = name[:name.rindex('-') + 1]

        self.options['field_options']['prefix'] = prefix

        attrs = attrs or {}
        attrs['data-location-field-options'] = json.dumps(self.options)

        # Django added renderer parameter in 1.11, made it mandatory in 2.1
        kwargs = {}
        if renderer is not None:
            kwargs['renderer'] = renderer
        text_input = super(LocationWidget, self).render(name, value, attrs=attrs, **kwargs)

        return render_to_string('location_field/map_widget.html', {
            'field_name': name,
            'field_input': mark_safe(text_input)
        })

    @property
    def media(self):
        return forms.Media(**settings.LOCATION_FIELD['resources.media'])



from django.conf import settings

LOCATION_FIELD_PATH = settings.STATIC_URL + 'location_field'

LOCATION_FIELD = {
    'map.provider': 'google',
    'map.zoom': 13,

    'search.provider': 'google',
    'search.suffix': '',

    # Google
    'provider.google.api': '//maps.google.com/maps/api/js',
    'provider.google.api_key': '',
    'provider.google.map_type': 'ROADMAP',

    # Mapbox
    'provider.mapbox.access_token': '',
    'provider.mapbox.max_zoom': 18,
    'provider.mapbox.id': 'mapbox.streets',

    # OpenStreetMap
    'provider.openstreetmap.max_zoom': 18,

    # misc
    'resources.root_path': LOCATION_FIELD_PATH,
    'resources.media': {
        'js': [
            LOCATION_FIELD_PATH + '/js/form.js',
        ],
    },
}



from django.apps import AppConfig
from django.conf import settings

from location_field.settings import LOCATION_FIELD


class DefaultConfig(AppConfig):
    name = 'location_field'
    verbose_name = 'Location Field'

    def ready(self):
        self.patch_settings()

    def patch_settings(self):
        config = LOCATION_FIELD.copy()
        config.update(getattr(settings, 'LOCATION_FIELD', {}))

        settings.LOCATION_FIELD = config



from django.test import TestCase
from django.conf import settings

from location_field.apps import DefaultConfig

from tests.models import Place
from tests.forms import LocationForm

from pyquery import PyQuery as pq

import json
import location_field


class LocationFieldTest(TestCase):
    def test_plain(self):
        vals = {
            'city': 'Bauru',
            'location': '-22.2878573,-49.0905487',
        }

        obj = Place.objects.create(**vals)

        self.assertEqual(obj.city, 'Bauru')
        self.assertEqual(obj.location, '-22.2878573,-49.0905487')

    def test_settings(self):
        with self.settings(LOCATION_FIELD={'map.provider': 'foobar'}):
            app_config = DefaultConfig('location_field', location_field)
            app_config.patch_settings()

            self.assertEqual(settings.LOCATION_FIELD.get('map.provider'),
                             'foobar')

    def test_field_options(self):
        form = LocationForm(initial={})
        d = pq(str(form))

        opts = json.loads(d('[data-location-field-options]').attr(
            'data-location-field-options'))

        location_field_opts = settings.LOCATION_FIELD

        for key, value in location_field_opts.items():
            self.assertEqual(value, opts[key])

    def test_custom_resources(self):
        form = LocationForm(initial={})

        self.assertIn('form.js', str(form.media))

        with self.settings(LOCATION_FIELD={
                'resources.media': {'js': ['foo.js', 'bar.js']}}):
            self.assertIn('foo.js', str(form.media))
            self.assertIn('bar.js', str(form.media))


if settings.TEST_SPATIAL:
    from . import spatial_test



from django.db import models

from location_field.models.spatial import LocationField


class SpatialPlace(models.Model):
    parent_place = models.ForeignKey('self', blank=True, null=True)
    city = models.CharField(max_length=255)
    location = LocationField(based_fields=['city'], zoom=7)
